
/* SEMsgProducer_dpm
 * A plugin that applies dpm-specifics to message sending
 *
 *
 *
 * by Fabrizio Furano, CERN, Feb 2010
 */


#include "SEMsgProducer_dpm.hh"
#include "../SimpleDebug.hh"
#include "dpns_api.h"
#include "stdlib.h"
#include "serrno.h"

// Dpm won't send this message, hence we short-circuit it
int SEMsgProducer_dpm::NotifyChmod(char *authDN, char *filename, int perm) {

  return 0;

}

// Dpm won't send this message, hence we short-circuit it
int SEMsgProducer_dpm::NotifyChown(char *authDN, char *filename, char *username, char *groupname) {

  return 0;

}

// Dpm won't send this message, hence we short-circuit it
int SEMsgProducer_dpm::NotifySetACL(char *authDN, char *filename, char *acl) {

  return 0;

}



int SEMsgProducer_dpm::NotifyNotAvailable(char *authDN, char *filename) {
  
  return SEMsgProducer::NotifyNotAvailable(authDN, filename);
}



int SEMsgProducer_dpm::NotifyAvailable(char *authDN, char *filename) {
  
  return SEMsgProducer::NotifyAvailable(authDN, filename);
}

int SEMsgProducer_dpm::NotifyDeleted(char *authDN, char *filename) {

  return SEMsgProducer::NotifyDeleted(authDN, filename);
}

int SEMsgProducer_dpm::NotifyAddReplica(char *authDN, 
                                char      *lfn,
				char      *guid,
				int        mode,
				long long  size,
				char      *csumtype,
				char      *csumvalue,
				char      *server,
				char      *sfn,
				char      *status,
				char      *f_type,
				char      *poolname,
				char      *fs) {

  // This is a DPM, hence there is no notion of lfn here. The fact that the same db and the same code
  // of LFC is used is very confusing.
  // Anyway, what is called 'lfn' here is what the LFC would call 'sfn'
  //
  // Remember, this notification must make sense, hence we call the things with their name in the reality.
  // Hence this notification will have empty lfn, and the GUID will be the one that was passed. It seems
  // that the guid field is never filled by the clients, but this questionable practice must not become
  // our problem
  return SEMsgProducer::NotifyAddReplica(authDN, "", guid, mode, size, csumtype, csumvalue, server, lfn, status,
					 f_type, poolname, fs);
}


//----------------------------
// The hook function to get an instance of the class
//  when the plugin is loaded
//----------------------------

extern "C" SEMsgProducer *GetSEMsgProducer(GetSEMsgProducerArgs)
{
   activemq::library::ActiveMQCPP::initializeLibrary();
   return (SEMsgProducer *)new SEMsgProducer_dpm(logobj, cfgobj, brokerURI, destname, useTopic, sessionTransacted);
}
