
/* SEMsgProducer_lfc
 * A plugin that applies lfc-specifics to message sending
 *
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 */


#include "SEMsgProducer_lfc.hh"
#include "../SimpleDebug.hh"
#include "lfc_api.h"
#include "stdlib.h"
#include "serrno.h"


int SEMsgProducer_lfc::NotifyChmod(char *authDN, char *filename, int perm) {
  // Loop on the replicas and notify them individually

    int flags;
    lfc_list list;
    struct lfc_filereplica *lp;
    
    flags = CNS_LIST_BEGIN;
    while ((lp = lfc_listreplica (filename, NULL, flags, &list)) != NULL) {
      flags = CNS_LIST_CONTINUE;
      /* process the entry */
      SEMsgProducer::NotifyChmod(authDN, lp->sfn, perm);
    }
    (void) lfc_listreplica (filename, NULL, CNS_LIST_END, &list);


  return 0;

}

int SEMsgProducer_lfc::NotifyChown(char *authDN, char *filename, char *username, char *groupname) {
  // Loop on the replicas and notify them individually

  int flags;
  lfc_list list;
  struct lfc_filereplica *lp;
    
  flags = CNS_LIST_BEGIN;
  while ((lp = lfc_listreplica (filename, NULL, flags, &list)) != NULL) {
    flags = CNS_LIST_CONTINUE;
    /* process the entry */
    SEMsgProducer::NotifyChown(authDN, lp->sfn, username, groupname);
  }
  (void) lfc_listreplica (filename, NULL, CNS_LIST_END, &list);
  

  return 0;

}


int SEMsgProducer_lfc::NotifySetACL(char *authDN, char *filename, char *acl) {
  // Loop on the replicas and notify them individually

    int flags;
    lfc_list list;
    struct lfc_filereplica *lp;
    
    flags = CNS_LIST_BEGIN;
    while ((lp = lfc_listreplica (filename, NULL, flags, &list)) != NULL) {
      flags = CNS_LIST_CONTINUE;
      /* process the entry */
      SEMsgProducer::NotifySetACL(authDN, lp->sfn, acl);
    }
    (void) lfc_listreplica (filename, NULL, CNS_LIST_END, &list);


  return 0;

}


// Lfc won't send this message, hence we short-circuit it
int SEMsgProducer_lfc::NotifyNotAvailable(char *authDN, char *filename) {

  return 0;
}

// Lfc won't send this message, hence we short-circuit it
int SEMsgProducer_lfc::NotifyAvailable(char *authDN, char *filename) {

  return 0;
}


int SEMsgProducer_lfc::NotifyDeleted(char *authDN, char *filename) {
  // Just notify based on the parameter.

  SEMsgProducer::NotifyDeleted(authDN, filename);

  return 0;
}



int SEMsgProducer_lfc::NotifyAddReplica(char *authDN, 
				char      *lfn,
				char      *guid,
				int        mode,
				long long  size,
				char      *csumtype,
				char      *csumvalue,
				char      *server,
				char      *sfn,
				char      *status,
				char      *f_type,
				char      *poolname,
				char      *fs) {

  return SEMsgProducer::NotifyAddReplica(authDN, lfn, guid, mode, size, csumtype, csumvalue, server, sfn, status,
					 f_type, poolname, fs);

}


//----------------------------
// The hook function to get an instance of the class
//  when the plugin is loaded
//----------------------------

extern "C" SEMsgProducer *GetSEMsgProducer(GetSEMsgProducerArgs)
{
   activemq::library::ActiveMQCPP::initializeLibrary();
   return (SEMsgProducer *)new SEMsgProducer_lfc(logobj, cfgobj, brokerURI, destname, useTopic, sessionTransacted);
}
