#ifndef SEMSGPRODUCERLFC_HH
#define SEMSGPRODUCERLFC_HH

/* SEMsgProducer_lfc
 * A plugin that applies lfc-specifics to message sending
 *
 *
 *
 * by Fabrizio Furano, CERN, Oct 2010
 */



#include "../SEMsgProducer.hh"




class SEMsgProducer_lfc : public SEMsgProducer {


public:
   SEMsgProducer_lfc( SimpleDebug *logobj,
		      Config *cfgobj,
		      const std::string& brokerURI,
		      const std::string& destname,
		      bool useTopic = false,
		      bool sessionTransacted = false ):
     SEMsgProducer(logobj, cfgobj, brokerURI, destname, useTopic, sessionTransacted) {

     char *fname = "SEMsgProducer_lfc::SEMsgProducer_lfc";

     // The LFC client needs these envvars to work
     // We don't want to mess up with the startup scripts, hence we get them
     // from the config file
     std::string s;
     s = CFG->GetString("prodapi.lfcplugin.host", "");
     if (s.length() > 0)
        setenv("LFC_HOST", s.c_str(), 1);
     Info(SimpleDebug::kLOW, fname, "LFC_HOST: " << getenv("LFC_HOST"));

     s = CFG->GetString("prodapi.cert", "");
     if (s.length() > 0)
       setenv("X509_USER_CERT", s.c_str(), 1);
     Info(SimpleDebug::kLOW, fname, "X509_USER_CERT: " << getenv("X509_USER_CERT"));

     s = CFG->GetString("prodapi.certkey", "");
     if (s.length() > 0)
       setenv("X509_USER_KEY", s.c_str(), 1);
     Info(SimpleDebug::kLOW, fname, "X509_USER_KEY: " << getenv("X509_USER_KEY"));

     setenv("CSEC_MECH", "ID", 1);
     Info(SimpleDebug::kLOW, fname, "CSEC_MECH: " << getenv("CSEC_MECH"));


};

   // Some examples of useful messages to override with lfc-related processing
   virtual int NotifyAvailable(char *authDN, char *filename);
   virtual int NotifyDeleted(char *authDN, char *filename);
   virtual int NotifyNotAvailable(char *authDN, char *filename);
   virtual int NotifyChmod(char *authDN, char *filename, int perm);
   virtual int NotifyChown(char *authDN, char *filename, char *username, char *groupname);
   virtual int NotifySetACL(char *authDN, char *filename, char * acl);
   virtual int NotifyAddReplica(char      *authDN, 
				char      *lfn,
				char      *guid,
				int        mode,
				long long  size,
				char      *csumtype,
				char      *csumvalue,
				char      *server,
				char      *sfn,
				char      *status,
				char      *f_type,
				char      *poolname,
				char      *fs);

  virtual int CanNotifyNotAvailable() { return false; };
  virtual int CanNotifyAvailable()    { return false; };
  virtual int CanNotifyDeleted()      { return CFG->GetBool("prodapi.lfcplugin.notifydeleted", true); };
  virtual int CanNotifyChmod()        { return CFG->GetBool("prodapi.lfcplugin.notifychmod", true); };
  virtual int CanNotifyChown()        { return CFG->GetBool("prodapi.lfcplugin.notifychown", true); };
  virtual int CanNotifySetACL()       { return CFG->GetBool("prodapi.lfcplugin.notifysetacl", true); };
  virtual int CanNotifyAddReplica()   { return CFG->GetBool("prodapi.lfcplugin.notifyaddreplica", false); };
};





#endif
