
/* SEMsgConsumer_pyintf
 * A plugin that invokes python funcs
 *
 *
 * by Fabrizio Furano, CERN, Mar 2011
 */

#include <Python.h>

#include "SEMsgConsumer_pyintf.hh"

#include "../SEMsgProtoDefs.hh"
#include "../SimpleDebug.hh"
#include <cms/MapMessage.h>

using namespace cms;



PyObject* log_CaptureStdout(PyObject* self, PyObject* pArgs)
{
 char* LogStr = NULL;
 if (!PyArg_ParseTuple(pArgs, "s", &LogStr)) return NULL;


 Info(SimpleDebug::kHIGH, "PythonStdout", LogStr);
 //printf("%s", LogStr); 
 // Simply using printf to do the real work. 
 // You could also write it to a .log file or whatever...
 // MessageBox(NULL, LogStr...
 // WriteFile(hFile, LogStr...

 Py_INCREF(Py_None);
 return Py_None;
}

// Notice we have STDERR too.
PyObject* log_CaptureStderr(PyObject* self, PyObject* pArgs)
{
 char* LogStr = NULL;
 if (!PyArg_ParseTuple(pArgs, "s", &LogStr)) return NULL;

 Info(SimpleDebug::kHIGH, "PythonStderr", LogStr);
 //printf("%s", LogStr);

 Py_INCREF(Py_None);
 return Py_None;
}

static PyMethodDef logMethods[] = {
 {"CaptureStdout", log_CaptureStdout, METH_VARARGS, "Logs stdout"},
 {"CaptureStderr", log_CaptureStderr, METH_VARARGS, "Logs stderr"},
 {NULL, NULL, 0, NULL}
};




SEMsgConsumer_pyintf::SEMsgConsumer_pyintf( const std::string& brokerURI,
					    const std::string& destname,
					    bool useTopic,
					    bool sessionTransacted ):
  SEMsgConsumer(brokerURI, destname, useTopic, sessionTransacted) {

  char *fname = "SEMsgConsumer_pyintf::SEMsgConsumer_pyintf";

  // Load from the cfg file the definitions of the scripts/funcs to call
  Py_Initialize();



  




  Py_InitModule("log", logMethods);
  PyRun_SimpleString(
		     "import log\n"
		     "import sys\n"
		     "class StdoutCatcher:\n"
		     "\tdef write(self, str):\n"
		     "\t\tlog.CaptureStdout(str)\n"
		     "class StderrCatcher:\n"
		     "\tdef write(self, str):\n"
		     "\t\tlog.CaptureStderr(str)\n"
		     "sys.stdout = StdoutCatcher()\n"
		     "sys.stderr = StderrCatcher()\n"
		     );























  // FileAvailable
  info_FileAvailable.module = CFG->GetString("consd.pyintfplugin.module_FileAvailable", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.module_FileAvailable: " << info_FileAvailable.module);

  info_FileAvailable.func = CFG->GetString("consd.pyintfplugin.func_FileAvailable", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.func_FileAvailable: " << info_FileAvailable.func);

  if (pyinit(info_FileAvailable))
    pyterm(info_FileAvailable);

  // FileNotAvailable
  info_FileNotAvailable.module = CFG->GetString("consd.pyintfplugin.module_FileNotAvailable", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.module_FileNotAvailable: " << info_FileNotAvailable.module);

  info_FileNotAvailable.func = CFG->GetString("consd.pyintfplugin.func_FileNotAvailable", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.func_FileNotAvailable: " << info_FileNotAvailable.func);

  if (pyinit(info_FileNotAvailable))
    pyterm(info_FileNotAvailable);


  // FileDeleted
  info_FileDeleted.module = CFG->GetString("consd.pyintfplugin.module_FileDeleted", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.module_FileDeleted: " << info_FileDeleted.module);

  info_FileDeleted.func = CFG->GetString("consd.pyintfplugin.func_FileDeleted", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.func_FileDeleted: " << info_FileDeleted.func);

  if (pyinit(info_FileDeleted))
    pyterm(info_FileDeleted);


  // AddReplica
  info_AddReplica.module = CFG->GetString("consd.pyintfplugin.module_AddReplica", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.module_AddReplica: " << info_AddReplica.module);

  info_AddReplica.func = CFG->GetString("consd.pyintfplugin.func_AddReplica", "");
  Info(SimpleDebug::kLOW, fname, "consd.pyintfplugin.func_AddReplica: " << info_AddReplica.func);

  if (pyinit(info_AddReplica))
    pyterm(info_AddReplica);
};


SEMsgConsumer_pyintf::~SEMsgConsumer_pyintf() throw(){
     PyErr_Clear();
     Py_Finalize();
}

void SEMsgConsumer_pyintf::onMessageFileNotAvailable( const cms::MapMessage* m ) {
  const char *fname="SEMsgConsumer_pyintf::onMessageFileNotAvailable";
  std::string fn;
  int retval = 0;
  std::vector<std::string> myargs;

  Info(SimpleDebug::kMEDIUM, fname, "Starting.");




  std::string sender = "Unknown sender";
  if (m->propertyExists(PROTO_SENDERDN))
    sender = m->getStringProperty(PROTO_SENDERDN);

  // Now invoke the Python API with the info we got, and do whatever we need to do

  int cnt = 1;
  if (m->propertyExists(PROTO_NENTRIES))
    cnt = m->getIntProperty(PROTO_NENTRIES);

  Info(SimpleDebug::kHIGH, fname, "NEntries=" << cnt);

  if ( (cnt == 1) && (getStringField(fn, m, PROTO_FILENOTAVAILABLE_FILENAME_S, -1, fname) > 0) ) {
    
    myargs.push_back(fn);
    retval = pyxeqfunc2(info_FileNotAvailable.pFunc, sender, myargs);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_FileNotAvailable.module << "', '" <<
	    info_FileNotAvailable.func << "', args) returned " << retval);
  }
  else {

    for (int i = 1; i < cnt+1; i++) {
      if (getStringField(fn, m, PROTO_FILENOTAVAILABLE_FILENAME_S, i, fname) > 0) {
	myargs.push_back(fn);
      }

    }
    retval = pyxeqfunc2(info_FileNotAvailable.pFunc, sender, myargs);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_FileNotAvailable.module << "', '" <<
	    info_FileNotAvailable.func << "', args) returned " << retval);



  }


}



void SEMsgConsumer_pyintf::onMessageFileDeleted( const cms::MapMessage* m ) {
  const char *fname="SEMsgConsumer_pyintf::onMessageFileDeleted";
  std::string fn;
  int retval = 0;
  std::vector<std::string> myargs;

  Info(SimpleDebug::kMEDIUM, fname, "Starting.");




  std::string sender = "Unknown sender";
  if (m->propertyExists(PROTO_SENDERDN))
    sender = m->getStringProperty(PROTO_SENDERDN);

  // Now invoke the Python API with the info we got, and do whatever we need to do

  int cnt = 1;
  if (m->propertyExists(PROTO_NENTRIES))
    cnt = m->getIntProperty(PROTO_NENTRIES);

  Info(SimpleDebug::kHIGH, fname, "NEntries=" << cnt);

  if ( (cnt == 1) && (getStringField(fn, m, PROTO_FILEDELETED_FILENAME_S, -1, fname) > 0) ) {
    
    myargs.push_back(fn);
    retval = pyxeqfunc2(info_FileDeleted.pFunc, sender, myargs);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_FileDeleted.module << "', '" <<
	    info_FileDeleted.func << "', args) returned " << retval);
  }
  else {

    for (int i = 1; i < cnt+1; i++) {
      if (getStringField(fn, m, PROTO_FILEDELETED_FILENAME_S, i, fname) > 0) {
	myargs.push_back(fn);
      }

    }
    retval = pyxeqfunc2(info_FileDeleted.pFunc, sender, myargs);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_FileDeleted.module << "', '" <<
	    info_FileDeleted.func << "', args) returned " << retval);


  }


}




void SEMsgConsumer_pyintf::onMessageFileAvailable( const cms::MapMessage* m ) {
  const char *fname="SEMsgConsumer_pyintf::onMessageFileAvailable";
  std::string fn;
  int retval = 0;
  std::vector<std::string> myargs;

  Info(SimpleDebug::kMEDIUM, fname, "Starting.");




  std::string sender = "Unknown sender";
  if (m->propertyExists(PROTO_SENDERDN))
    sender = m->getStringProperty(PROTO_SENDERDN);

  // Now invoke the Python API with the info we got, and do whatever we need to do

  int cnt = 1;
  if (m->propertyExists(PROTO_NENTRIES))
    cnt = m->getIntProperty(PROTO_NENTRIES);

  Info(SimpleDebug::kHIGH, fname, "NEntries=" << cnt);

  if ( (cnt == 1) && (getStringField(fn, m, PROTO_FILEAVAILABLE_FILENAME_S, -1, fname) > 0) ) {
    
    myargs.push_back(fn);
    retval = pyxeqfunc2(info_FileAvailable.pFunc, sender, myargs);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_FileAvailable.module << "', '" <<
	    info_FileAvailable.func << "', args) returned " << retval);
  }
  else {

    for (int i = 1; i < cnt+1; i++) {
      if (getStringField(fn, m, PROTO_FILEAVAILABLE_FILENAME_S, i, fname) > 0) {
	myargs.push_back(fn);
      }

    }
    retval = pyxeqfunc2(info_FileAvailable.pFunc, sender, myargs);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_FileAvailable.module << "', '" <<
	    info_FileAvailable.func << "', args) returned " << retval);



  }


}




void SEMsgConsumer_pyintf::onMessageAddReplica( const cms::MapMessage* m ) {
  const char *fname="SEMsgConsumer_pyintf::onMessageAddReplica";

  std::string lfn, guid, csumtype, csumvalue, server, sfn;
  long long size;
  long mode;
  char strsize[64];
  char strmode[64];

  int retval = 0;
  std::vector<std::string> myargs;

  Info(SimpleDebug::kMEDIUM, fname, "Starting.");




  std::string sender = "Unknown sender";
  if (m->propertyExists(PROTO_SENDERDN))
    sender = m->getStringProperty(PROTO_SENDERDN);

  // Now invoke the Python API with the info we got, and do whatever we need to do

  int cnt = 1;
  if (m->propertyExists(PROTO_NENTRIES))
    cnt = m->getIntProperty(PROTO_NENTRIES);

  Info(SimpleDebug::kHIGH, fname, "NEntries=" << cnt);

  if ( (cnt == 1) &&
       (getStringField(lfn,         m, PROTO_ADDREPLICA_LFN_S, -1, fname) > 0) && 
       (getStringField(guid,        m, PROTO_ADDREPLICA_GUID_S, -1, fname) >= 0) && 
       (getLongField(mode,          m, PROTO_ADDREPLICA_MODE_L, -1, fname) >= 0) &&
       (getLongLongField(size,      m, PROTO_ADDREPLICA_SIZE_LL, -1, fname) >= 0) &&
       (getStringField(csumtype,    m, PROTO_ADDREPLICA_CSUMTYPE_S, -1, fname) >= 0) &&
       (getStringField(csumvalue,   m, PROTO_ADDREPLICA_CSUMVALUE_S, -1, fname) >= 0) &&
       (getStringField(server,      m, PROTO_ADDREPLICA_SERVER_S, -1, fname) >= 0) && 
       (getStringField(sfn,         m, PROTO_ADDREPLICA_SFN_S, -1, fname) > 0) ) {
    
    myargs.push_back(lfn);
    myargs.push_back(guid);
    sprintf(strmode, "%ld", mode);
    myargs.push_back(strmode);
    sprintf(strsize, "%lld", size);
    myargs.push_back(strsize);
    myargs.push_back(csumtype);
    myargs.push_back(csumvalue);
    myargs.push_back(server);
    myargs.push_back(sfn);

    retval = pyxeqfunc2(info_AddReplica.pFunc, sender, myargs, 8);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_AddReplica.module << "', '" <<
	    info_AddReplica.func << "', args) returned " << retval);
  }
  else {

    for (int i = 1; i < cnt+1; i++) {
      if ( (getStringField(lfn,         m, PROTO_ADDREPLICA_LFN_S,   i, fname) > 0) && 
       (getStringField(guid,        m, PROTO_ADDREPLICA_GUID_S,      i, fname) >= 0) && 
       (getLongField(mode,          m, PROTO_ADDREPLICA_MODE_L,      i, fname) >= 0) &&
       (getLongLongField(size,      m, PROTO_ADDREPLICA_SIZE_LL,     i, fname) >= 0) &&
       (getStringField(csumtype,    m, PROTO_ADDREPLICA_CSUMTYPE_S,  i, fname) >= 0) &&
       (getStringField(csumvalue,   m, PROTO_ADDREPLICA_CSUMVALUE_S, i, fname) >= 0) &&
       (getStringField(server,      m, PROTO_ADDREPLICA_SERVER_S,    i, fname) >= 0) && 
       (getStringField(sfn,         m, PROTO_ADDREPLICA_SFN_S,       i, fname) > 0) ) {

	myargs.push_back(lfn);
	myargs.push_back(guid);
	sprintf(strmode, "%ld", mode);
	myargs.push_back(strmode);
	sprintf(strsize, "%lld", size);
	myargs.push_back(strsize);
	myargs.push_back(csumtype);
	myargs.push_back(csumvalue);
	myargs.push_back(server);
	myargs.push_back(sfn);

      }

    }
    retval = pyxeqfunc2(info_AddReplica.pFunc, sender, myargs, 8);

    if (retval)
      Error(fname, "pyxeqfunc2('" << info_AddReplica.module << "', '" <<
	    info_AddReplica.func << "', args) returned " << retval);



  }


}



void logpythonerror(char *fname) {
  PyObject* ptype;
  PyObject* pvalue;
  PyObject* ptraceback;
  PyObject* exc = NULL;
  PyObject* pstr;
  char *str = NULL;

  PyErr_Fetch(&ptype, &pvalue, &ptraceback);
  PyErr_NormalizeException(&ptype, &pvalue, &ptraceback);
  pstr = PyObject_Str(ptype);

  int lineno = -1;
  if (ptraceback) lineno = ((PyTracebackObject*)ptraceback)->tb_lineno;
  std::string errmsg = "(null)";
  if (pvalue) {
    str = PyString_AsString(pvalue);
    if (str) errmsg = str;
  }

  Error(fname, "Error '" << errmsg << "' occurred on line: " << lineno << " - " << PyString_AsString(pstr));

  Py_XDECREF(ptype);
  Py_XDECREF(pvalue);
  Py_XDECREF(ptraceback);
  Py_XDECREF(exc);

  PyErr_Clear();
}


int SEMsgConsumer_pyintf::pyxeqfunc(std::string &module, std::string &func, std::string sender, std::vector<std::string> &args)
{
  char *fname = "SEMsgConsumer_pyintf::pyxeqfunc";

  PyObject *pName, *pModule, *pFunc;
  PyObject *pArgs, *pValue, *pArgs2;

  int i;
  std::string func2 = func;

  Py_Initialize();
  Info(SimpleDebug::kHIGH, fname, "PYTHONPATH: " << getenv("PYTHONPATH"));

  pName = PyString_FromString(module.c_str());
  /* Error checking of pName left out */
  
  pModule = PyImport_Import(pName);
  Py_DECREF(pName);

  if (pModule != NULL) {
    pFunc = PyObject_GetAttrString(pModule, (char *)func2.c_str());
    /* pFunc is a new reference */
    
    if (pFunc && PyCallable_Check(pFunc)) {
      pArgs = PyTuple_New(2);

      // First arg: the original msender
      pValue = PyString_FromString(sender.c_str());
      PyTuple_SetItem(pArgs, 0, pValue);

      // Second arg: all the 'files' related parameter in a single tuple
      pArgs2 = PyTuple_New(args.size());
      for (i = 0; i < (int)args.size(); ++i) {
	pValue = PyString_FromString(args[i].c_str());
	if (!pValue) {
	  Py_DECREF(pArgs);
	  Py_DECREF(pModule);
	  Error(fname, "Cannot convert argument " << i << ":" << args[i]);
	  return 1;
	}
        /* pValue reference stolen here: */
        PyTuple_SetItem(pArgs2, i, pValue);
    
      }

      /* pValue reference stolen here: */
      PyTuple_SetItem(pArgs, 1, pArgs2);


    

    Info(SimpleDebug::kHIGHEST, fname, "Invoking " << func2);
    pValue = PyObject_CallObject(pFunc, pArgs);
    Py_DECREF(pArgs);
    if (pValue != NULL) {
      Info(SimpleDebug::kHIGH, fname, "Result of call: " << PyInt_AsLong(pValue));
      Py_DECREF(pValue);
    }
    else {
      Py_DECREF(pFunc);
      Py_DECREF(pModule);

      logpythonerror(fname);
      PyErr_Print();

      Error(fname, "Call failed: " << func2);
      return 1;
    }
  }
  else {
    if (PyErr_Occurred()) {
      logpythonerror(fname);
      PyErr_Print();
    }
    Error(fname, "Cannot find function " << func2);
  }
  Py_XDECREF(pFunc);
  Py_DECREF(pModule);
}
 else {
   logpythonerror(fname);

   PyErr_Print();
   Error(fname, "Failed to load " << module << " Have you checked the current PYTHONPATH?");
   return 1;
 }

  PyErr_Clear();
  Py_Finalize();
return 0;
}








// Initialize all the python vars that are needed to invoke the given function efficiently
int SEMsgConsumer_pyintf::pyinit(myPyFuncInfo &funcnfo)
{
  char *fname = "SEMsgConsumer_pyintf::pyinit";
  PyObject *pName;

  if (funcnfo.module == "") return 1;
  if (funcnfo.func == "") return 1;

  Info(SimpleDebug::kHIGHEST, fname, "PYTHONPATH: " << getenv("PYTHONPATH"));

  // Dirty trick against sloppy Python scripts
  char tmpbuf[1024];
  char *p = tmpbuf;
  strcpy(tmpbuf, funcnfo.module.c_str());
  strcat(tmpbuf, ".py");
  PySys_SetArgv(1, &p);


  pName = PyString_FromString(funcnfo.module.c_str());
  /* Error checking of pName left out */
  
  funcnfo.pModule = PyImport_Import(pName);
  //Py_DECREF(pName);

  if (funcnfo.pModule != NULL) {

    funcnfo.pFunc = PyObject_GetAttrString(funcnfo.pModule, (char *)funcnfo.func.c_str());
    /* pFunc is a new reference */
    if (funcnfo.pFunc && PyCallable_Check(funcnfo.pFunc)) {
      return 0;
    }
    else {
      if (PyErr_Occurred())
	logpythonerror(fname);

      PyErr_Clear();
      Error(fname, "Cannot find function '" << funcnfo.func << "' in module '" << funcnfo.module << "'.");
      return 1;
    }

  }
  else {
    if (PyErr_Occurred())
      logpythonerror(fname);

    PyErr_Clear();
    Error(fname, "Failed to load Python module '" << funcnfo.module << "'. Have you checked the current PYTHONPATH? " << getenv("PYTHONPATH"));
    return 1;
  }

}



// Initialize all the python vars that are needed to invoke the given function efficiently
int SEMsgConsumer_pyintf::pyterm(myPyFuncInfo &funcnfo)
{
  //char *fname = "SEMsgConsumer_pyintf::pyterm";

  if (funcnfo.pFunc) Py_XDECREF(funcnfo.pFunc);
  funcnfo.pFunc = NULL;

  if (funcnfo.pModule) Py_DECREF(funcnfo.pModule);
  funcnfo.pModule = NULL;


return 0;
}



int SEMsgConsumer_pyintf::pyxeqfunc2(PyObject *pFunc, std::string sender, std::vector<std::string> &args, int argsplit)
{
  char *fname = "SEMsgConsumer_pyintf::pyxeqfunc2";

  PyObject *pArgs, *pValue, *pArgs2;

  if (pFunc && PyCallable_Check(pFunc)) {
    pArgs = PyTuple_New(1+argsplit);

    // First arg: the original msender
    pValue = PyString_FromString(sender.c_str());
    PyTuple_SetItem(pArgs, 0, pValue);

    // Second, third... arg: all the args related parameters, each one in a single tuple
    for (int j = 1; j <= argsplit; j++) {
      pArgs2 = PyTuple_New(args.size()/argsplit);

      
      for (int i = 0, cnt=0; i < (int)args.size(); i += argsplit, cnt++) {
	pValue = PyString_FromString(args[i].c_str());
	if (!pValue) {
	  if (PyErr_Occurred())
	    logpythonerror(fname);

	  Py_DECREF(pArgs);
	  Py_DECREF(pArgs2);
	  PyErr_Clear();
	  Error(fname, "Cannot convert argument " << i << ":" << args[i]);
	  return 1;
	}

	/* pValue reference stolen here: */
	PyTuple_SetItem(pArgs2, cnt, pValue);
      
      }

      /* pArgs2 reference stolen here: */
      PyTuple_SetItem(pArgs, j, pArgs2);
    }

    

    Info(SimpleDebug::kHIGHEST, fname, "Invoking func");

    pValue = PyObject_CallObject(pFunc, pArgs);
    Py_DECREF(pArgs);
    
    if (pValue != NULL) {
      Info(SimpleDebug::kHIGH, fname, "Result of call: " << PyInt_AsLong(pValue));
      Py_DECREF(pValue);
    }
    else {
      if (PyErr_Occurred())
	logpythonerror(fname);

      
      Error(fname, "Call failed.");
      return 1;
    }
  }
    

  PyErr_Clear();

  return 0;
}














//----------------------------
// The hook function to get an instance of the class
//  when the plugin is loaded
//----------------------------

extern "C" SEMsgConsumer *GetSEMsgConsumer(GetSEMsgConsumerArgs)
{
  return (SEMsgConsumer *)new SEMsgConsumer_pyintf(brokerURI, destname, useTopic, sessionTransacted);
}
