#ifndef SEMSGCONSUMERPYINTF_HH
#define SEMSGCONSUMERPYINTF_HH

/* SEMsgConsumer_pyintf
 * A plugin that invokes python funcs
 *
 *
 * by Fabrizio Furano, CERN, Mar 2011
 */

#include "../SEMsgConsumer.hh"
#include <string>
#include <vector>

// Helper mini-class that holds values useful to talk to Python, related to a function to be invoked
class myPyFuncInfo {
public:
  std::string module, func;
  PyObject *pModule, *pFunc;
  myPyFuncInfo(): pModule(NULL), pFunc(NULL) {}
};

// Subclass that overrides the events with lfc-specific calls
class SEMsgConsumer_pyintf: public SEMsgConsumer {
private:
  myPyFuncInfo info_FileNotAvailable, info_FileDeleted, info_FileAvailable, info_AddReplica;

  // All in one version
  int pyxeqfunc(std::string &module, std::string &func, std::string sender, std::vector<std::string> &args);

  // Optimized version
  int pyinit(myPyFuncInfo &funcnfo);
  int pyterm(myPyFuncInfo &funcnfo);
  int pyxeqfunc2(PyObject *pFunc, std::string sender, std::vector<std::string> &args, int argsplit=1);

public:

   SEMsgConsumer_pyintf( const std::string& brokerURI,
			 const std::string& destname,
			 bool useTopic = false,
			 bool sessionTransacted = false );

  virtual ~SEMsgConsumer_pyintf() throw();

   // These do what they say
   virtual void onMessageFileAvailable( const cms::MapMessage* m );
   virtual void onMessageFileDeleted( const cms::MapMessage* m );
   virtual void onMessageFileNotAvailable( const cms::MapMessage* m );
   virtual void onMessageAddReplica( const cms::MapMessage* m );
};


#endif
